@@ -138,7 +138,6 @@ class MetadataTypeResource(object):
 
         _LOG.info("Updating metadata type %s", metadata_type.name)
 
-
         with self._db.connect() as connection:
             connection.update_metadata_type(
                 name=metadata_type.name,
@@ -1038,15 +1037,18 @@ class DatasetResource(object):
             for dataset in self._make_many(connection.search_datasets_by_metadata(metadata)):
                 yield dataset
 
-    def search(self, **query):
+    def search(self, limit=None, **query):
         """
         Perform a search, returning results as Dataset objects.
 
         :param dict[str,str|float|Range] query:
+        :param int limit:
         :rtype: __generator[Dataset]
         """
         source_filter = query.pop('source_filter', None)
-        for _, datasets in self._do_search_by_product(query, source_filter=source_filter):
+        for _, datasets in self._do_search_by_product(query,
+                                                      source_filter=source_filter,
+                                                      limit=limit):
             for dataset in self._make_many(datasets):
                 yield dataset
 
@@ -1174,8 +1176,11 @@ class DatasetResource(object):
             q['dataset_type_id'] = product.id
             yield q, product
 
+    # pylint: disable=too-many-locals
     def _do_search_by_product(self, query, return_fields=False, select_field_names=None,
-                              with_source_ids=False, source_filter=None):
+                              with_source_ids=False, source_filter=None,
+                              limit=None):
+
         if source_filter:
             product_queries = list(self._get_product_queries(source_filter))
             if not product_queries:
@@ -1192,36 +1197,38 @@ class DatasetResource(object):
             source_exprs = None
 
         product_queries = list(self._get_product_queries(query))
-        with self._db.connect() as connection:
-            for q, product in product_queries:
-                dataset_fields = product.metadata_type.dataset_fields
-                query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
-                select_fields = None
-                if return_fields:
-                    # if no fields specified, select all
-                    if select_field_names is None:
-                        select_fields = tuple(field for name, field in dataset_fields.items()
-                                              if not field.affects_row_selection)
-                    else:
-                        select_fields = tuple(dataset_fields[field_name]
-                                              for field_name in select_field_names)
+        for q, product in product_queries:
+            dataset_fields = product.metadata_type.dataset_fields
+            query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
+            select_fields = None
+            if return_fields:
+                # if no fields specified, select all
+                if select_field_names is None:
+                    select_fields = tuple(field for name, field in dataset_fields.items()
+                                          if not field.affects_row_selection)
+                else:
+                    select_fields = tuple(dataset_fields[field_name]
+                                          for field_name in select_field_names)
+            with self._db.connect() as connection:
                 yield (product,
                        connection.search_datasets(
                            query_exprs,
                            source_exprs,
                            select_fields=select_fields,
+                           limit=limit,
                            with_source_ids=with_source_ids
                        ))
 
     def _do_count_by_product(self, query):
         product_queries = self._get_product_queries(query)
-        with self._db.connect() as connection:
-            for q, product in product_queries:
-                dataset_fields = product.metadata_type.dataset_fields
-                query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
+
+        for q, product in product_queries:
+            dataset_fields = product.metadata_type.dataset_fields
+            query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
+            with self._db.connect() as connection:
                 count = connection.count_datasets(query_exprs)
-                if count > 0:
-                    yield product, count
+            if count > 0:
+                yield product, count
 
     def _do_time_count(self, period, query, ensure_single=False):
         if 'time' not in query:
@@ -1240,10 +1247,10 @@ class DatasetResource(object):
                 raise ValueError('Multiple products match single query search: %r' %
                                  ([dt.name for q, dt in product_queries],))
 
-        with self._db.connect() as connection:
-            for q, product in product_queries:
-                dataset_fields = product.metadata_type.dataset_fields
-                query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
+        for q, product in product_queries:
+            dataset_fields = product.metadata_type.dataset_fields
+            query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
+            with self._db.connect() as connection:
                 yield product, list(connection.count_datasets_through_time(
                     start,
                     end,
